# (c) 2002 Nuxeo SARL <http://nuxeo.com>
# (c) 2002 Florent Guillaume <mailto:fg@nuxeo.com>
# (c) 2002 Julien Jalon <mailto:jj@nuxeo.com>
# (c) 2002 Prfecture du Bas-Rhin, France
# (c) 2002 CIRB, Belgique
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: SkinnerDocument.py,v 1.6 2002/11/19 15:56:14 fermigier Exp $

"""
  Special documents that skin Hierarchies
"""

import string
from string import join
from urllib import quote

from cpsdocument_utils import getImageForFile
from BaseDocument import BaseDocument, BaseDocument_adder
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import aq_base

from Products.CMFCore.CMFCorePermissions import setDefaultRoles, \
     View, ModifyPortalContent

# debuging stuff
from zLOG import LOG, INFO, DEBUG
def log(var, msg='log : '):
  LOG(msg, DEBUG, var)



base_factory_type_information = (
    {'id': 'Base Skinner Document',
     'description': 'A document that skins supported containers.',
     'meta_type': 'Base Skinner Document',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addBaseSkinnerDocument',
     'immediate_view': 'baseskinnerdocument_edit_form',
     'allow_discussion': 0,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'baseskinnerdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'baseskinnerdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'baseskinnerdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': ()},
                 )
     },
    )

# Base for all skinner documents
class BaseSkinnerDocument(BaseDocument):
    """
    Base class for documents
    """

    meta_type = 'Base Skinner Document'

    security = ClassSecurityInfo()

    isSkinner = 1
    isPrincipiaFolderish = 0


    def manage_afterAdd(self, item, container):
        """Registers item in the container
        """
        LOG('SKINNER', INFO, 'Adding "%s" to "%s"' % (getattr(item, 'id', '???'), getattr(container,'id', '???')))
        BaseDocument.manage_afterAdd(self, item, container)
        if item is self:
            LOG('SKINNER', INFO, "That's me!")
            container.skinnerdocument = self.id

    def manage_beforeDelete(self, item, container):
        """Eventually unregisters item from container
        """
        LOG('SKINNER', INFO, 'Removing "%s" from "%s"' % (getattr(item, 'id', '???'), getattr(container,'id', '???')))
        BaseDocument.manage_beforeDelete(self, item, container)
        if item is self:
            LOG('SKINNER', INFO, "That's me!")
            if container.skinnerdocument is self.id:
                LOG('SKINNER', INFO, "And I'm the skinner!")
                LOG('SKINNER', INFO, "Looking for an other skinner")
                del container.skinnerdocument
                items = container.objectItems()
                items.reverse()
                for id, document in items:
                    if aq_base(getattr(document, 'isSkinner', 0)):
                        LOG('SKINNER', INFO, "Found %s" % id)
                        container.skinnerdocument = id
                        break

    security.declareProtected(View, 'skin_document')
    def skin_container(self, container, item, doc_infos):
        return "Abstract skinner (%s)" % (self.title_or_id(), )

    security.declareProtected(ModifyPortalContent, 'setSkin')
    def setSkin(self, container):
        if getattr(container, 'skinnerdocument', None) != self.id:
            LOG('SKINNER', INFO, 'Changing skin from %s to %s' % (container.id, self.id))
            container.skinnerdocument = self.id

InitializeClass(BaseSkinnerDocument)


def addBaseSkinnerDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a Base Skinner Document"""
    ob = BaseSkinnerDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

# Concrete skinner class. Simply uses 
factory_type_information = (
    {'id': 'Skinner Document',
     'description': 'A document that skins supported containers with a template.',
     'title': '_portal_type_Skinner Document',
     'meta_type': 'Skinner Document',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addSkinnerDocument',
     'immediate_view': 'skinnerdocument_edit_form',
     'allow_discussion': 0,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'skinnerdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'skinnerdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'skinnerdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': ()},
                 )
     },
    )

# for all skinner documents
class SkinnerDocument(BaseSkinnerDocument):
    """
    class for documents.
    """

    meta_type = 'Skinner Document'

    security = ClassSecurityInfo()

    _properties = BaseSkinnerDocument._properties + (
        {'id':'template', 'type':'text', 'mode':'w', 'label':'Template'},
        {'id':'champfaq', 'type':'string', 'mode':'w', 'label':'body of the hierarchy'},
        {'id':'champnouvelle', 'type':'string', 'mode':'w', 'label':'body of the hierarchy'},
        )

    template = ''
    champfaq=''
    champnouvelle=''  

    security.declareProtected(View, 'skin_document')
    def skin_container(self, container, item, doc_infos):
        template = self.template
        if template:
            render = getattr(self, template, None)
            if render is not None:
                try:
                    if getattr(aq_base(render), 'isDocTemp', 0):
                        return render(self, self.REQUEST, container=container, item=item, doc_infos=doc_infos)
                    else:
                        return render(container=container, item=item, doc_infos=doc_infos)
                except:
                    # XXX show the error stack ?
                    raise
                    return "Error in template %s" % (template, )
            else:
                return "Template %s not found" % (template, )
        else:
            return "No template provided"

    security.declareProtected(View, 'addOneResult')
    def addOneResult(self,REQUEST=None):
        "Method for add just one result or more"

        if REQUEST:
            form = REQUEST.form
        else:
            return 0
          
        checked = form.get('option1')
        champ = form.get('champ')
        n = 0

        if len(checked)<2:
            return self.reviewedhierarchylevel_searchForm()

        brains = self.search_get2(self.REQUEST)

        if champ=='champnouvelle':
            goodBrains = [x for x in brains if x['title'] in checked]
            newlist = [{'title': x['title'],
                        'url': x['url'],
                        'brain': x['brain'],
                        'newsdate':x['newsdate'],
                        'moddate':x['modification'],
                        'description':x['description'],
                        'Burl':  '/'.join(x['url'].split('//')[-1].split('/')[2:]),                     } for x in goodBrains]

            log(newlist, 'new list : ')
            s= """\n\n<!-- begin of result search -->\n"""

            for b in newlist:
                try:
                    asImg = b['brain'].getObject().vignette1
                except AttributeError:
                    asImg = ''
                log(asImg, 'asImg : ') 
                Img = ''
                if asImg: 
                    Img = """<img src="%s/vignette1" width="120" height="72" 
                                  alt="vignette1" border="0" 
                                  vspace="5">""" % (b['Burl'],)

                log('Img : ', Img)
                s = s + """
<table cellpadding="0" cellspacing="0">
  <tr>
    <td width="6"><img src="blank.gif" width="6" height="1" 
                       alt="" border="0"></td>
    <td width="24" valign="top"><img src="puce_double.gif" width="24" 
        height="10" alt="" border="0" vspace="2"></td>
    <td width="160" valign="top">
      <span class="cssTexteCouleur1"><b>%s</b></span></td></tr>
    <tr>
      <td colspan="3">
        <table border="0" cellpadding="0" cellspacing="0" width="190">
          <tr>
            <td width="15"> <img src="blank.gif" width="15" height="1" 
                                 alt="" border="0">
            </td>
            <td width="175"><span class="cssTitreActu">%s <br></span>
              <a href="%s" class="cssLinkTexteDefault">%s [&nbsp;Lire&nbsp; 
                <img src="plus.gif" width="6" height="10" alt="" border="0" 
                     align="absmiddle">&nbsp;]<br></a>
                %s <!-- L'image  -->     
            </td>
          </tr>
        </table>
      </td>
    </tr>
    <tr>
      <td colspan="3"><img src="blank.gif" width="1" height="5" alt="" 
                           border="0"></td>
    </tr>
    <tr>
      <td colspan="2" width="1"><img src="blank.gif" width="1" height="1" 
                                     alt="" border="0"></td>
      <td background="tiret_horizontal.gif">
        <img src="blank.gif" width="1" height="1" alt="" border="0"></td> 
    </tr>
    <tr>
      <td colspan="3">&nbsp;</td>
    </tr>
</table>
<!-- end of result search -->
""" % (b['newsdate'], b['title'], b['Burl'], b['description'][:100], Img)

            #s="<span class=cssTexteCouleur1><b>"+b['newsdate']+"</b></span><br><b>"+b['title']+"</b><br><a class=cssLinkTexteDefault href="+ Burl +">"+b['description']+"&nbsp;[Lire<b>+</b>]</a><br><br><img src="+ Burl+ "/vignette1><br>"

            self.champnouvelle=self.champnouvelle+s

        elif champ =='champfaq':
            list = []
            for b in brains :
                if b['title'] == checked[n]:
                    n=n+1
                    list.append({'title':b['title'],
                               'url':b['url'],
                               'moddate':b['modification'],
                               #'answer':b['answer'],
                               #'answer':b['brain'].getObject().answer[:100],
                               'description':b['description']})
            s= """\n\n<!-- begin of result search -->\n"""
            for b in list:
                Burl = '/'.join(b['url'].split('//')[-1].split('/')[2:])
                
                s = s + """
<table cellpadding="0" cellspacing="0">
  <tr>
    <td width="24" valign="top"><img 
        src="puce_double.gif" width="24" height="10" 
        alt="" border="0" vspace="2"></td>
    <td width="200" valign="top" colspan="2"> 
    

<span class=cssTexteCouleur1>
  <b>Question : </b>
</span>
<span class=cssTexteDefault> %s </span><br>
<span class=cssTexteCouleur1>Rponse : </span>
<a class=cssLinkTexteDefault href=%s> 
   %s &nbsp;[Lire
</a>
<b style="{color: #03B245;}">+</b>
]<br><br>

    </td >
  </tr> 

  <tr>
    <td colspan="2" width="1"><img src="blank.gif" width="1" height="1" 
                         alt="" border="0"></td>
    <td background="tiret_horizontal.gif"><img 
        src="blank.gif" width="1" height="1" alt="" border="0"></td>
</tr>
</table>

<!-- end of result search -->
""" % (b['title'], Burl, b['description'][:100])

                self.champfaq=self.champfaq+s

        return self.skinnerdocument_edit_form() 


    security.declareProtected(View, 'search_get2')
    def search_get2(self, REQUEST=None):
        "like search_get + date mod"

        if REQUEST:
            form = REQUEST.form
        else:
            return 0
        search = {}
        st = form.get('SearchableText', '')
        if st:
            search['SearchableText'] = st
        t = form.get('Theme')
        search['review_state'] = "published"

        f =form.get('portal_type')
        if f:
           search['meta_type']=f

        brains = self.search_get(search)

        ru=form.get('rubrique')
        if ru:
           lbrains = []
           for dico in brains :
             cc=dico['public_categories']
             for y in cc:
               if ru in cc:
                 lbrains.append(dico)
           return lbrains

        v = form.get('date_mod')
        if not v:
            return brains
        dtv = 0
        if v and match(r'^[0-9]?[0-9]/[0-9]?[0-9]/[0-9]{4,4}$', v):
            locale = self.portal_messages.get_selected_language()
            if locale == 'fr':
                d, m, y = v.split('/')
            else:
                m, d, y = v.split('/')

            try:
                dtv = DateTime(int(y), int(m), int(d), 0, 0)
            except:
                return brains
        ret=[]
        for b in brains:
            if DateTime(b['moddate']) > dtv:
                ret.append(b)
        return ret




InitializeClass(SkinnerDocument)


def addSkinnerDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a Skinner Document"""
    ob = SkinnerDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

