#
# (C) Copyright 2002 Nuxeo SARL <contact@nuxeo.com>
#
# This product is governed by a license. See LICENSE.txt file.
#

# Zope
import Globals
from zLOG import LOG, DEBUG, INFO
from Post import Post
from DateTime import DateTime
from interfaces.Forum import IForum
from AccessControl import ClassSecurityInfo
from OFS.Image import Image
from Acquisition import aq_base

# Python
from types import *

# CMF
from Products.CMFCore.utils              import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, \
     AddPortalContent, setDefaultRoles

# CPS
from Products.NuxCPSDocuments.cpsdocument_utils import getImageForFile
from Products.NuxWorkgroup.Workgroup import Workgroup, ManageWorkgroups
from Products.NuxCPSDocuments.BaseDocument import BaseDocument

# Forum Moderator Permissions
ManageForumContent = 'Manage Forum Content'
setDefaultRoles(ManageForumContent, ('Manager', 'WorkgroupManager'))


factory_type_information = (
  {'id': 'Forum',
   'meta_type': 'Forum',
   'description': "Forums hold threaded discussions",
   'icon': 'icon_post.gif',
   'product': 'NuxCPSForum',
   'factory': 'addForum',
   'filter_content_types': 1,
   'allowed_content_types': ('Post',),
   'immediate_view': 'forum_edit_form',
   'actions': (
                {
                  'id': 'view',
                  'name': '_action_view_',
                  'action': 'forum_listing',
                  'permissions': (View,),
                }, {
                  'id': 'localroles',
                  'name': '_action_access_rights_',
                  'action': 'workgroup_localrole_form',
                  'permissions': (ManageWorkgroups,),
                  'category': 'object',
                }, {
                  'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'forum_edit_form',
                  'permissions': (ManageProperties,),
                }, {
                  'id': 'forum_post_message',
                  'name': '_action_forum_post_',
                  'action': 'forum_post_message',
                  'permissions': (AddPortalContent,),
                }, {
                  'id': 'subscribe',
                  'name': '_action_subscribe',
                  'action': 'forum_notification',
                  'visible': 0,
                  'permissions': (View,)
                }, {
                  'id': 'unsubscribe',
                  'name': '_action_unsubscribe',
                  'action': 'forum_notification',
                  'visible': 0,
                  'permissions': (View,)
                }, {
                  'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': (),
                },
              )
 },
)

class Forum(Workgroup):

    meta_type = 'Forum'
    __implements__ = IForum

    security = ClassSecurityInfo()

    _properties = Workgroup._properties + (
        {'id': 'attachments_allowed', 'type': 'boolean', 'mode': 'w', 
         'label': 'Attachments allowed in posts'},
    )

    isDocumentContainer = 0
    isPrincipiaFolderish = 0

    # Default attributes
    isprivate = 0
    notification_on = 1
    attachments_allowed = 1
    isprio_moderation = 1
    ispost_moderation = 1
    subscription_allowed = 1
    anonymous_subscription = 0
    flatten = 0
    sorting = 'date'
    msg_per_page = 15
    image = None
    
    def __init__(self, id, title=''):
        """Constructor."""
        Workgroup.__init__(self, id, title=title)

    def manage_afterAdd(self, item, container):
        """Correctly set Forum permissions."""
        Workgroup.manage_afterAdd(self, item, container)
        if item is self:
            self.manage_permission('Delete objects', 
                ['Manager', 'WorkgroupManager'], 0)
            self.manage_permission('Modify portal content', 
                ['Manager', 'WorkgroupManager', 'Owner'], 0)
    
    security.declareProtected(ManageProperties, 'edit')    
    def edit(self, title='', description=''):
        """Edition method from ancestor."""
        Workgroup.edit(self, title=title, description=description)

#         #XXX: should be written in the workflow
#         forumBrain = getToolByName(self, 'portal_catalog')({"id":self.getId()})[0]
#         container = self.aq_inner.aq_parent
#         if not forumBrain["PublicCategories"] or (not container.getId() in forumBrain["PublicCategories"])
#             forumBrain["PublicCategories"].append(container.getId())
# 
        
    security.declareProtected(ManageProperties, 'setForumProperties')    
    def setForumProperties(self, isprivate, attachments_allowed,
                           isprio_moderation, ispost_moderation,
                           subscription_allowed, anonymous_subscription,
                           flatten=0,
                           sorting='date', 
						   msg_per_page=15, image=None,
                           image_change='keep'):
        """Set the Forum properties for the instance."""
        #
        # Forum preferences
        #
        self.isprivate = isprivate
        self.attachments_allowed = attachments_allowed
        self.isprio_moderation = isprio_moderation
        self.ispost_moderation = ispost_moderation
        self.subscription_allowed = subscription_allowed
        self.anonymous_subscription = anonymous_subscription
        self.flatten = flatten
        self.sorting = sorting
        self.msg_per_page = msg_per_page
        if image_change == 'change':
            if image is not None:
                self.image = getImageForFile(image=image)
        elif image_change == 'delete':
            self.image = None

    security.declareProtected(View, 'getUserName')    
    def getUserName(self):
        """Return the user's login name."""
        mtool = getToolByName(self, 'portal_membership')
        return mtool.getAuthenticatedMember().getMemberId()
    
    security.declareProtected(View, 'isModerator')    
    def isModerator(self):
        """Return wether the current user can manage this forum."""
        mtool = getToolByName(self, 'portal_membership')
        return mtool.checkPermission(ManageProperties, self)
        
    security.declareProtected(View, 'listPosts')    
    def listPosts(self):
        """Return the top-level posts."""
        mtool = getToolByName(self, 'portal_membership')
        posts = [ child for child in self.contentValues(['Post'])
                        if mtool.checkPermission(View, child) ]
        return posts

    children = listPosts

    security.declareProtected(View, 'getForum')    
    def getForum(self):
        """Return the Forum object."""
        return self

    security.declareProtected(View, 'getForumImage')
    def getForumImage(self):
        """Return the Forum image."""
        return self.image

    security.declareProtected(View, 'getMail')    
    def getMail(self, login):
        """Return the user mail reference to login."""
        # Use the CPS MetaDirectories
        members = getToolByName(self, 'portal_metadirectories').members
        t = members.getEntry(login)
        if t is not None:
            return t['email']
        else:
            return ""

    #
    # Subscription
    #
    security.declareProtected(View, 'isSubscribed')
    def isSubscribed(self, REQUEST=None):
        """Return the user subscription state."""
        stool = getToolByName(self, "portal_subscription")
        return stool.isSubscribed(stool.getMemberOrMail(REQUEST),self)

    security.declareProtected(View, 'subscribe')
    def subscribe(self, REQUEST=None):
        """Return the user subscription state."""
        stool = getToolByName(self, "portal_subscription")
        return stool.subscribe(stool.getMemberOrMail(REQUEST),self)

    security.declareProtected(View, 'subscribe')
    def unsubscribe(self, REQUEST=None):
        """Return the user subscription state."""
        stool = getToolByName(self, "portal_subscription")
        return stool.unsubscribe(stool.getMemberOrMail(REQUEST),self)

    security.declareProtected(View, 'isSubscribed')
    def isSubscribed(self, REQUEST=None):
        """Return the user subscription state."""
        stool = getToolByName(self, "portal_subscription")
        return stool.isSubscribed(stool.getMemberOrMail(REQUEST),self)

    security.declareProtected(View, 'subscribe')
    def getSubscriber(self, REQUEST=None):
        """Subscribe user to Post"""
        return getToolByName(self, "portal_subscription").getMemberOrMail(REQUEST)

    security.declareProtected(View, 'listSubscriptionsFor')
    def listSubscriptionsFor(self, REQUEST=None):
        """Return observed objects for current user."""
        stool = getToolByName(self, "portal_subscription")
        return [ object for object 
                 in stool.listObservedObjectsFor(
                     stool.getMemberOrMail(REQUEST) )
                 if object.portal_type 
                     in (self.portal_type, 'Post') ]

    #
    # Snapshots
    #
    security.declareProtected(AddPortalContent, 'createSnapShot')
    def createSnapShot(self, id, title, thread):
        """Create a thread snapshot in this Forum's container."""
        container = self.aq_inner.aq_parent
        container.invokeFactory('ThreadSnapShot', id=id, title=title,
                                original_thread=thread, flatten=self.flatten)
        return getattr(container, id)

    def log(message,summary='',severity=0):
        LOG('NuxCPSForum',severity,summary,'>>%s<<\n' %str(message) + '\n')

def addForum(self, id, title='', REQUEST=None):
    """Add a Forum object."""
    forum = Forum(id, title)
    self._setObject(id, forum)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

Globals.InitializeClass(Forum)
